/**
 *
 *  Copyright Openmind http://www.openmindonline.it
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package net.sourceforge.openutils.mgnlcriteria.jcr.query.criterion;

import java.util.Calendar;

import net.sourceforge.openutils.mgnlcriteria.jcr.query.Criteria;
import net.sourceforge.openutils.mgnlcriteria.jcr.query.JCRQueryException;

/**
 * @author fgrilli
 * @version $Id: BetweenExpression.java 1193 2009-05-08 21:05:22Z fgrilli $
 */
public class BetweenExpression implements Criterion
{

    private static final long serialVersionUID = 6686395240415024541L;

    private final String propertyName;

    private final Object lo;

    private final Object hi;

    protected BetweenExpression(String propertyName, Object lo, Object hi)
    {
        this.propertyName = propertyName;
        this.lo = lo;
        this.hi = hi;
    }

    @Override
    public String toString()
    {
        return propertyName + " between " + lo + " and " + hi;
    }

    public String toXPathString(Criteria criteria) throws JCRQueryException
    {
        StringBuilder fragment = new StringBuilder();
        fragment.append(" (").append(propertyName).append(" >= ");

        if (lo instanceof String && hi instanceof String)
            fragment.append("'" + lo + "' and " + propertyName + " <= '" + hi + "'");
        else if (lo instanceof Number && hi instanceof Number)
            fragment.append(lo + " and " + propertyName + " <= " + hi);
        else if (lo instanceof Calendar && hi instanceof Calendar)
        {
            Calendar cal = (Calendar) lo;
            Calendar cal2 = (Calendar) hi;
            StringBuilder date = new StringBuilder();
            date.append(XS_DATETIME_FUNCTION + "('");
            date
                .append(cal.get(Calendar.YEAR))
                .append(HYPHEN)
                .append(
                    cal.get(Calendar.MONTH) < MONTH_MAX
                        ? "0" + (cal.get(Calendar.MONTH) + 1)
                        : cal.get(Calendar.MONTH) + 1)
                .append(HYPHEN)
                .append(
                    cal.get(Calendar.DAY_OF_MONTH) < DAY_MAX ? "0" + cal.get(Calendar.DAY_OF_MONTH) : cal
                        .get(Calendar.DAY_OF_MONTH));
            date.append(MIDNIGHT);
            date.append("') ");

            StringBuilder date2 = new StringBuilder();
            date2.append(XS_DATETIME_FUNCTION + "('");
            date2.append(cal2.get(Calendar.YEAR)).append(HYPHEN).append(
                cal2.get(Calendar.MONTH) < MONTH_MAX
                    ? "0" + (cal2.get(Calendar.MONTH) + 1)
                    : cal2.get(Calendar.MONTH) + 1).append(HYPHEN).append(
                cal2.get(Calendar.DAY_OF_MONTH) < DAY_MAX ? "0" + cal2.get(Calendar.DAY_OF_MONTH) : cal2
                    .get(Calendar.DAY_OF_MONTH));
            date2.append(T235959);
            date2.append("') ");
            fragment.append(date.toString() + " and " + propertyName + " <= " + date2.toString());
        }
        else
        {
            String msg = "values provided are not of the accepted types String, Number, Calendar";
            log.error(msg);
            throw new IllegalArgumentException(msg);
        }
        fragment.append(")");
        log.debug("xpathString is {} ", fragment);
        return fragment.toString();
    }
}
